% lab16_ex_transforms.m
% 2D orthogonal trasforms of images
clear all; close all;

% Initialization - read image
 [x,cmap] = imread('Cameraman.tif');   % read image to "x", color palette to "cmap"
%[x,cmap] = imread('Elaine.tiff');     % read image to "x", color palette to "cmap"
[M, N] = size(x);                      % ask for number of rows and columns
figure; imshow(x,cmap), title('Image'); pause  % display image using its palette

x = double(x);                         % pixel values to double (gray scale)
figure; hist(x(:),50); title('Histogram'); pause  % histogram of image pixel values

% Filter masks
% Low-pass smoothing
hLP1 = 1/9* [ 1 1 1; 1 1 1; 1 1 1];
hLP2 = 1/10*[ 1 1 1; 1 2 1; 1 1 1];
hLP3 = 1/16*[ 1 2 1; 2 4 2; 1 2 1];
hLP4 = 1/273*[ 1 4 7 4 1; 4 16 26 16 4; 7 26 41 26 7; 4 16 26 16 4;  1 4 7 4 1 ];
% Horizontal and vertical edge detection:
hS1 = [ 1 2 1; 0 0 0; -1 -2 -1 ];       % Sobel 1
hS2 = [ 1 0 -1; 2 0 -2; 1 0 -1];        % Sobel 2
hP1 = [ 1 1 1; 0 0 0; -1 -1 -1];        % Prewitt 1
hP2 = [ 1 0 -1; 1 0 -1; 1 0 -1];        % Prewitt 2
% Diagonal edge detection:
hR1 = [ 1 0; 0 -1 ];                    % Roberts 1
hR2 = [ 0 1; -1 0 ];                    % Roberts 2
hG1 = [ 1 1 1; 1 -2 -1; 1 -1 -1 ];      % gradient 1
hG2 = [ 1 1 1; -1 -2 1; -1 -1 1 ];      % gradient 2
% Double differentiation - edge detection in all directions:
hL1 = [ 0 -1 0; -1 4 -1; 0 -1 0 ];      % laplasian 1
hL2 = [ 1 -2 1; -2 4 -2; 1 -2 1 ];      % laplasjan 2
hL3 = [ -1 -1 -1; -1 8 -1; -1 -1 -1 ];  % laplasjan 3
hL4 = [ -1 -1 -1 -1 -1; -1 -1 -1 -1 -1; -1 -1 24 -1 -1; -1 -1 -1 -1 -1; -1 -1 -1 -1 -1 ];
hL5 = [ 0 0 1 0 0; 0 1 2 1 0; 1 2 -16 2 1; 0 1 2 1 0; 0 0 1 0 0];
hL6 = [ 0 1 1 2 2 2 1 1 0; 1 2 4 5 5 5 4 2 1; 1 4 5 3 0 3 5 4 1;...
        2 5 3 -12 -24 -12 3 5 2; ...
        2 5 0 -24 -40 -24 0 5 2; ...
        2 5 3 -12 -24 -12 3 5 2; ...
        1 4 5 3 0 3 5 4 1; 1 2 4 5 5 5 4 2 1; 0 1 1 2 2 2 1 1 0 ];

% 2D FILTERING using 2D Matlab convolution function conv2()
y = conv2(x, hLP3,'same');
figure;
subplot(121); imshow(x,cmap); title('BEFORE filter');
subplot(122); imshow(y,cmap); title('AFTER  filter'); pause

y1 = conv2(y, hS1,'same'); y2 = conv2(y, hS2,'same'); y12 = sqrt(y1.^2 + y2.^2); 
figure;
subplot(121); imshow(y,cmap); title('AFTER filter');
subplot(122); imshow(y12,cmap); title('AFTER edge'); pause
figure;
subplot(111); hist(y12(:),50); title('Histogram'); pause

% Binarization, morphological processing
[ixy] = find( y12 > 100 );                     % binarization
z = zeros(M,N); z(ixy) = 255*ones(size(ixy));  %
figure; imshow(z,cmap); title('AFTER binarization'); pause

z = imerode( z,  [ 1 1 1; 1 0 1; 1 1 1] );     % erosion
figure; imshow(z,cmap); title('AFTER erosion'); pause

z = imdilate( z, [ 1 1 1; 1 0 1; 1 1 1] );     % dilation
figure; imshow(z,cmap); title('AFTER dilation'); pause