% lab16_ex_image_demo.m
% Image processing demo - Image Processing Toolbox is required
% Find Matlab demo images: C:\Program Files\MATLAB\Rxxxxab\toolbox\images\imdata
% Use images Lena.BMP and Friends.JPG/Friends.PNG from the lecture
% In Octave: >>pkg load signal image[ENTER]
clear all; close all;

% Image reading and color treatment #######################################
% Initialization - read image (x) and color palette (cmap) if colors are indexed
  [x,cmap] = imread('bike.jpg');       % color (gray-scale), RGB MxNx3, cmap=empty
% [x,cmap] = imread('cameraman.tif');  % BW, MxNx1, cmap=empty
% [x,cmap] = imread('corn.tif');       % color, MxNx1, size(cmap)=256x3=equal=[0,1]?
% [x,cmap] = imread('kids.tif');       % color, MxNx1, size(cmap)=256x3=equal=[0,1]?
% [x,cmap] = imread('trees.tif');      % color, MxNx1, size(cmap)=256x3=equal=[0,1]?
% [x,cmap] = imread('peppers.png');    % color, RGB MxNx3, cmap=empty
% [x,cmap] = imread('hands1.jpg');     % color, RGB MxNx3, cmap=empty
% [x,cmap] = imread('office_4.jpg');   % color, RGB MxNx3, cmap=empty
% [x,cmap] = imread('Lena2.bmp');      % BW, MxNx1, size(cmap)=256x3=equal=[0,1]   

disp('On input:');
pal = size(cmap), [M, N, K] = size(x), % palette size, M=rows, N=cols, K=color planes
xmin = min(min(x)), xmax = max(max(x)),             % min, max pixel value
x_copy = x;                                         % copy image
cmap_copy = cmap;                                   % copy palette 
    figure; imshow(x,cmap), title('Image'); pause   % display image using its palette

if( ~isempty(cmap) & K==1 ) x=ind2gray(x,cmap); cmap = []; end   % for indexed color TIF
if(  isempty(cmap) & K==3 ) % for true color, RGB = no cmap, three color planes = RGB 
    if(1)  x = rgb2gray(x); % 3 color components --> color indexes, gray palette 
    else   x = x(:,:,2);    % choose only one color: 1=R, 2=G, 3=B
    end
end
disp('On output:'); pal=size(cmap), [M, N, K]=size(x), % M=rows, N=cols, K=color planes
    figure; imshow(x,cmap), title('Image'); pause      % display image using its palette
    disp('Please, rotate the image matrix!');          % display image as mesh  
    figure; mesh(double(x)); title('Image'); pause     % and rotate it manually

% IMAGE 2D DCT AND FILTERING using 2D DCT #################################
x = double( x );                                % integer values to double ones
xmin = min(min(x)), xmax = max(max(x)), pause   % min/max = ?
[m,n] = meshgrid(1:N,1:M); MN=max(M,N);         % filter mask indexes m,n
H(1:M,1:N) = exp(-(m.^2+n.^2)/(0.075*MN)^2);    % LP filter mask values in 2D-DCT domain
%H = ones(M,N) - H;                             % HP filter mask values in 2D-DCT domain                         
X1 = dct2( x ); % <============================ % 2D-DCT Matlab - image analysis
X2 = dct( dct( x ).' ).';                       % 2D-DCT as a sequence of 1D-DCT
err_dct = max(max(abs(X1-X2))),                 % error
X = X1; Xmin = min(min(X)), Xmax = max(max(X)), pause   % DCT spectrum min/max
X = X.*H;  % <================================= % 2D-DCT image spectrum modification
y = idct2( X ); % <============================ % 2D-IDCT - Matlab - image synthesis    
    figure;
    subplot(221); imshow(x, cmap);          title('Image');
    subplot(222); imshow(scaledB(X), cmap); title('2D DCT');
    subplot(223); imshow(scaledB(H), cmap); title('Filter mask');
    subplot(224); imshow(y, cmap);          title('Filtered image'); pause
  
% 2D FILTERING using 2D convolution #######################################
% low-pass:                                       hLP1,...mhLP4,
% edge -|\/ (Sobel, Prewitt, Roberts, Gradient):  hS1,hS2,hP1,hP2,hR1,hR2,hG1,hG2
% double differentiation (Laplacians):            hL1,...,hL6
filterweights                  % include file (from lecture) with different filter weights
y = conv2(x, hLP4,'same');     % low-pass filter
    figure;
    subplot(121); imshow(x,cmap); title('BEFORE filter');
    subplot(122); imshow(y,cmap); title('AFTER  filter'); pause
y1 = conv2(y, hS1,'same');     % filter Sobel1, Prewitt 1 --, Robets 1 //
y2 = conv2(y, hS2,'same');     % filter Sobel2, Prewitt 2 ||, Robets 2 \\
y12 = sqrt(y1.^2 + y2.^2);     % add two images = edges in all directions
    figure;
    subplot(221); imshow(x,cmap);   title('BEFORE filter');
    subplot(222); imshow(y1,cmap);  title('AFTER filter -- or //');
    subplot(223); imshow(y12,cmap); title('AFTER filter |-');
    subplot(224); imshow(y2,cmap);  title('AFTER filter || or \\'); pause

% Binarization, morphological processing ##################################
% z = imbinarize( y12 );                          % Matlab function  #1
% level = graythresh(y12); z = im2bw(y12,level);  % Matlab functions #2
[ixy] = find( y12 > 100 );                        % # our binarization
z = zeros(M,N); z(ixy) = 255*ones(size(ixy));     % # 
    figure; subplot(131); imshow(z,cmap); title('AFTER binarization');
z = imerode( z,  [ 1 1 1; 1 0 1; 1 1 1] );        % erosion
    subplot(132); imshow(z,cmap); title('AFTER erosion');
z = imdilate( z, [ 1 1 1; 1 0 1; 1 1 1] );        % dilation
    subplot(133); imshow(z,cmap); title('AFTER dilation'); pause

% Image Studio - mixing images ############################################    
% This part is correct only for RGB true color JPG/PNG images, e.g. "hand1.jpg"
figure; hist(x(:),50); title('Histogram'); pause
THR = 155; % threshold correct for hand1.jpg image, udjust it for others images
if( 0 )    % one color background for chosen image, at present yellow R=G=1,B=0 
   y(:,:,1) = 255*ones(M,N);  % Red
   y(:,:,2) = 255*ones(M,N);  % Green
   y(:,:,3) =   0*ones(M,N);  % Blue
   y = uint8( y ); 
else       % some other image is used as a background
  [y,cmap] = imread('Friends.png'); y = y(end-M+1:end,1:N,:);
end
for m=1:M
    for n=1:N
        if( x(m,n) < THR ) y(m,n,:) =  x_copy(m,n,:); end
    end
end
    figure; imshow(y); title('Image mixing'); pause
