% lab16_ex_watermarking.m
clear all; close all;

% Parameters
K = 32;     % block size for one watermark bit (K x K pixels)
gain = 1;   % watermark gain
  
% Read image to be watermarked
A = imread('lena2.bmp');
B = double(A); [M, N] = size(B);
  
% Addition of watermark
Mb = (M/K); Nb=(N/K);                 % number of blocks in row and column
plusminus1 = sign( randn(1,Mb*Nb) );  % random sequence of numbers +1/-1
Mark = zeros( size(B) );              % watermark: chessboard with +1/-1 pattern
for i = 1:Mb
    for j = 1:Nb
        Mark( (i-1)*K+1 : i*K, (j-1)*K+1 : j*K ) = plusminus1(i*j);   
    end
end
Noise = round( randn(size(B)) );      % noise (modulating signal)
MarkNoise = gain * Noise .* Mark;     % watermark = gain * noise * mark(+/-1)
B = uint8( B + MarkNoise );           % image + watermark, conversion to 8 bits
  
% Figures
figure, subplot(1,2,1), imshow(Mark,[]);      title('Watermark')
        subplot(1,2,2), imshow(MarkNoise,[]); title('Watermark * noise') 
figure, subplot(1,2,1); imshow(A,[]);         title('Original image')
        subplot(1,2,2); imshow(B,[]);         title('Image with watermark')
  
% Watermark detection
B = double(B);                              % conversion to double precision
 
% High-pass filtering
L = 10; L2=2*L+1;                           % 2D filter size (L x L)
w = hamming(L2); w = w * w';                % 2D Hamming window
   
f0=0.5; wc = pi*f0; [m n] = meshgrid(-L:L,-L:L);                       % 2D 
lp = wc * besselj(1, wc*sqrt(m.^2 + n.^2) )./(2*pi*sqrt(m.^2 + n.^2)); % low-pass
lp(L+1,L+1)= wc^2/(4*pi);                                              % filter
hp = - lp; hp(L+1,L+1) = 1 - lp(L+1,L+1);   % high-pass filter without window
h = hp .* w;                                % with 2D window
B = conv2( B, h, 'same');                   % image filtering
  
% Bit (+1/-1) detection in each pixel block of the watermark 
Demod = B .* Noise;                         % noise demodulation
MarkDetect = zeros( size(B) );              % pixel summation in blocks
for i=1:Mb
    for j=1:Nb
        MarkDetect((i-1)*K+1:i*K, (j-1)*K+1:j*K) = ...
               sign( sum( sum( Demod((i-1)*K+1:i*K, (j-1)*K+1:j*K) ) ) );
    end
end
errors = sum(sum( abs(Mark-MarkDetect) ))
  
% Figures
figure, subplot(1,2,1); imshow(Demod,[]);      title('Demodulation')
        subplot(1,2,2); imshow(MarkDetect,[]); title('Detection')
